@@ -1,4 +1,4 @@
-# $Id: W3C.pm 510 2005-09-12 18:13:03Z struan $
+# $Id: W3C.pm 559 2006-02-25 19:28:14Z struan $
 
 package WebService::Validator::HTML::W3C;
 
@@ -11,11 +11,11 @@ use WebService::Validator::HTML::W3C::Error;
 
 __PACKAGE__->mk_accessors(
     qw( http_timeout validator_uri _http_method
-      is_valid num_errors uri _content ) );
+      is_valid num_errors uri _content _output ) );
 
 use vars qw( $VERSION $VALIDATOR_URI $HTTP_TIMEOUT );
 
-$VERSION       = 0.10;
+$VERSION       = 0.11;
 $VALIDATOR_URI = 'http://validator.w3.org/check';
 $HTTP_TIMEOUT  = 30;
 
@@ -80,6 +80,14 @@ contacting the validator. By default this is 30 seconds.
 
 This fetches the XML response from the validator in order to provide information for the errors method. You should set this to true if you intend to use the errors method.
 
+=item output
+
+Controls which output format is used. Can be either xml or soap12.
+
+At the moment the default is XML as this is the only one supported by the Validator. However in the future it is moving to using SOAP for the detailed reporting.
+
+The default will always work so unless you're using a development version of the Validator you can safely ignore this.
+
 =back 
 
 =cut
@@ -100,6 +108,7 @@ sub _init {
     $self->http_timeout( $args{http_timeout}   || $HTTP_TIMEOUT );
     $self->validator_uri( $args{validator_uri} || $VALIDATOR_URI );
     $self->_http_method( $args{detailed} ? 'GET' : 'HEAD' );
+    $self->_output( $args{output} || 'xml' );
 }
 
 =head2 validate
@@ -267,18 +276,31 @@ sub errors {
     }
 
     my $xp       = XML::XPath->new( xml => $self->_content() );
-    my @messages = $xp->findnodes('/result/messages/msg');
 
-    foreach my $msg (@messages) {
-        my $err = WebService::Validator::HTML::W3C::Error->new(
-                                    {
-                                      line => $msg->getAttribute('line'),
-                                      col  => $msg->getAttribute('col'),
-                                      msg  => $msg->getChildNode(1)->getValue(),
-                                    }
-                                    );
+    if ( $self->_output eq 'xml' ) {
+        my @messages = $xp->findnodes('/result/messages/msg');
 
-        push @errs, $err;
+        foreach my $msg (@messages) {
+            my $err = WebService::Validator::HTML::W3C::Error->new({
+                          line => $msg->getAttribute('line'),
+                          col  => $msg->getAttribute('col'),
+                          msg  => $msg->getChildNode(1)->getValue(),
+                      });
+
+            push @errs, $err;
+        }
+    } else { # assume soap...
+       my @messages = $xp->findnodes( '/env:Envelope/env:Body/m:markupvalidationresponse/m:errors/m:errorlist/m:error' );
+
+       foreach my $msg ( @messages ) {
+           my $err = WebService::Validator::HTML::W3C::Error->new({ 
+                          line => $xp->find( './m:line', $msg )->get_node(1)->getChildNode(1)->getValue,
+                          col  => $xp->find( './m:col', $msg )->get_node(1)->getChildNode(1)->getValue,
+                          msg  => $xp->find( './m:message', $msg )->get_node(1)->getChildNode(1)->getValue,
+                      });
+
+            push @errs, $err;
+        }
     }
 
     return \@errs;
@@ -364,7 +386,7 @@ sub _construct_uri {
 
     # creating the HTTP query string with all parameters
     my $req_uri =
-      join ( '', "?uri=", uri_escape($uri_to_validate), ";output=xml" );
+      join ( '', "?uri=", uri_escape($uri_to_validate), ";output=", $self->_output );
 
     return $self->validator_uri . $req_uri;
 }
@@ -388,14 +410,14 @@ sub _get_request {
             return POST $self->validator_uri, 
                         Content_Type  =>  'form-data', 
                         Content       =>  [
-                                           output => 'xml',
+                                           output => $self->_output,
                                            uploaded_file => [ $uri->{ file } ],
                                           ];
         } elsif ( $uri->{ markup } ) {
             return POST $self->validator_uri, 
                         Content_Type  =>  'form-data', 
                         Content       =>  [
-                                           output => 'xml',
+                                           output => $self->_output,
                                            fragment => $uri->{ markup },
                                           ];
         }
